@@ -1,5 +1,5 @@
 /* DWARF2 EH unwinding support for MIPS Linux.
-   Copyright (C) 2004, 2005 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2006, 2007 Free Software Foundation, Inc.
 
 This file is part of GCC.
 
@@ -31,6 +31,7 @@ Boston, MA 02110-1301, USA.  */
    state data appropriately.  See unwind-dw2.c for the structs.  */
 
 #include <signal.h>
+#include <asm/unistd.h>
 
 /* The third parameter to the signal handler points to something with
  * this structure defined in asm/ucontext.h, but the name clashes with
@@ -51,7 +52,7 @@ mips_fallback_frame_state (struct _Unwind_Context *context,
 {
   u_int32_t *pc = (u_int32_t *) context->ra;
   struct sigcontext *sc;
-  _Unwind_Ptr new_cfa;
+  _Unwind_Ptr new_cfa, reg_offset;
   int i;
 
   /* 24021061 li v0, 0x1061 (rt_sigreturn)*/
@@ -59,49 +60,59 @@ mips_fallback_frame_state (struct _Unwind_Context *context,
   /*    or */
   /* 24021017 li v0, 0x1017 (sigreturn) */
   /* 0000000c syscall  */
-  if (*(pc + 1) != 0x0000000c)
+  if (pc[1] != 0x0000000c)
     return _URC_END_OF_STACK;
-  if (*(pc + 0) == 0x24021017)
+#if _MIPS_SIM == _ABIO32
+  if (pc[0] == (0x24020000 | __NR_sigreturn))
     {
       struct sigframe {
-	u_int32_t  trampoline[2];
+	u_int32_t ass[4];  /* Argument save space for o32.  */
+	u_int32_t trampoline[2];
 	struct sigcontext sigctx;
-      } *rt_ = context->ra;
+      } *rt_ = context->cfa;
       sc = &rt_->sigctx;
     }
-  else if (*(pc + 0) == 0x24021061)
+  else
+#endif
+  if (pc[0] == (0x24020000 | __NR_rt_sigreturn))
     {
       struct rt_sigframe {
-	u_int32_t  trampoline[2];
+	u_int32_t ass[4];  /* Argument save space for o32.  */
+	u_int32_t trampoline[2];
 	struct siginfo info;
 	_sig_ucontext_t uc;
-      } *rt_ = context->ra;
+      } *rt_ = context->cfa;
       sc = &rt_->uc.uc_mcontext;
     }
   else
     return _URC_END_OF_STACK;
 
-  new_cfa = (_Unwind_Ptr)sc;
-  fs->cfa_how = CFA_REG_OFFSET;
-  fs->cfa_reg = STACK_POINTER_REGNUM;
-  fs->cfa_offset = new_cfa - (_Unwind_Ptr) context->cfa;
+  new_cfa = (_Unwind_Ptr) sc;
+  fs->regs.cfa_how = CFA_REG_OFFSET;
+  fs->regs.cfa_reg = STACK_POINTER_REGNUM;
+  fs->regs.cfa_offset = new_cfa - (_Unwind_Ptr) context->cfa;
 
-#if _MIPS_SIM == _ABIO32 && defined __MIPSEB__
   /* On o32 Linux, the register save slots in the sigcontext are
      eight bytes.  We need the lower half of each register slot,
      so slide our view of the structure back four bytes.  */
-  new_cfa -= 4;
+#if _MIPS_SIM == _ABIO32 && defined __MIPSEB__
+  reg_offset = 4;
+#else
+  reg_offset = 0;
 #endif
 
   for (i = 0; i < 32; i++) {
     fs->regs.reg[i].how = REG_SAVED_OFFSET;
     fs->regs.reg[i].loc.offset
-      = (_Unwind_Ptr)&(sc->sc_regs[i]) - new_cfa;
+      = (_Unwind_Ptr)&(sc->sc_regs[i]) + reg_offset - new_cfa;
   }
-  fs->regs.reg[SIGNAL_UNWIND_RETURN_COLUMN].how = REG_SAVED_OFFSET;
-  fs->regs.reg[SIGNAL_UNWIND_RETURN_COLUMN].loc.offset
-    = (_Unwind_Ptr)&(sc->sc_pc) - new_cfa;
-  fs->retaddr_column = SIGNAL_UNWIND_RETURN_COLUMN;
+  /* The PC points to the faulting instruction, but the unwind tables
+     expect it point to the following instruction.  We compensate by
+     reporting a return address at the next instruction. */
+  fs->regs.reg[DWARF_ALT_FRAME_RETURN_COLUMN].how = REG_SAVED_VAL_OFFSET;
+  fs->regs.reg[DWARF_ALT_FRAME_RETURN_COLUMN].loc.offset
+    = (_Unwind_Ptr)(sc->sc_pc) + 4 - new_cfa;
+  fs->retaddr_column = DWARF_ALT_FRAME_RETURN_COLUMN;
 
   return _URC_NO_REASON;
 }
