@@ -28,14 +28,20 @@ extern bool hook_bool_void_false (void);
 extern bool hook_bool_void_true (void);
 extern bool hook_bool_bool_false (bool);
 extern bool hook_bool_mode_false (enum machine_mode);
-extern bool hook_bool_mode_rtx_false (enum machine_mode, rtx);
-extern bool hook_bool_mode_rtx_true (enum machine_mode, rtx);
+extern bool hook_bool_mode_const_rtx_false (enum machine_mode, const_rtx);
+extern bool hook_bool_mode_const_rtx_true (enum machine_mode, const_rtx);
 extern bool hook_bool_tree_false (tree);
+extern bool hook_bool_const_tree_false (const_tree);
 extern bool hook_bool_tree_true (tree);
-extern bool hook_bool_tree_hwi_hwi_tree_false (tree, HOST_WIDE_INT, HOST_WIDE_INT,
-					tree);
-extern bool hook_bool_tree_hwi_hwi_tree_true (tree, HOST_WIDE_INT, HOST_WIDE_INT,
-				       tree);
+extern bool hook_bool_const_tree_true (const_tree);
+extern bool hook_bool_const_tree_hwi_hwi_const_tree_false (const_tree,
+							   HOST_WIDE_INT,
+							   HOST_WIDE_INT,
+							   const_tree);
+extern bool hook_bool_const_tree_hwi_hwi_const_tree_true (const_tree,
+							  HOST_WIDE_INT,
+							  HOST_WIDE_INT,
+							  const_tree);
 extern bool hook_bool_rtx_false (rtx);
 extern bool hook_bool_uintp_uintp_false (unsigned int *, unsigned int *);
 extern bool hook_bool_rtx_int_int_intp_false (rtx, int, int, int *);
@@ -50,8 +56,8 @@ extern void hook_void_FILEptr_constcharptr (FILE *, const char *);
 extern void hook_void_tree (tree);
 extern void hook_void_tree_treeptr (tree, tree *);
 
-extern int hook_int_tree_0 (tree);
-extern int hook_int_tree_tree_1 (tree, tree);
+extern int hook_int_const_tree_0 (const_tree);
+extern int hook_int_const_tree_const_tree_1 (const_tree, const_tree);
 extern int hook_int_rtx_0 (rtx);
 extern int hook_int_size_t_constcharptr_int_0 (size_t, const char *, int);
 extern int hook_int_void_no_regs (void);
@@ -62,16 +68,16 @@ extern tree hook_tree_tree_tree_bool_null (tree, tree, bool);
 
 extern unsigned hook_uint_uint_constcharptrptr_0 (unsigned, const char **);
 
-extern bool default_can_output_mi_thunk_no_vcall (tree, HOST_WIDE_INT,
-					   HOST_WIDE_INT, tree);
+extern bool default_can_output_mi_thunk_no_vcall (const_tree, HOST_WIDE_INT,
+						  HOST_WIDE_INT, const_tree);
 
 extern rtx hook_rtx_rtx_identity (rtx);
 extern rtx hook_rtx_rtx_null (rtx);
 extern rtx hook_rtx_tree_int_null (tree, int);
 
-extern const char *hook_constcharptr_tree_null (tree);
-extern const char *hook_constcharptr_rtx_null (rtx);
-extern const char *hook_constcharptr_tree_tree_null (tree, tree);
-extern const char *hook_constcharptr_int_tree_null (int, tree);
-extern const char *hook_constcharptr_int_tree_tree_null (int, tree, tree);
+extern const char *hook_constcharptr_const_tree_null (const_tree);
+extern const char *hook_constcharptr_const_rtx_null (const_rtx);
+extern const char *hook_constcharptr_const_tree_const_tree_null (const_tree, const_tree);
+extern const char *hook_constcharptr_int_const_tree_null (int, const_tree);
+extern const char *hook_constcharptr_int_const_tree_const_tree_null (int, const_tree, const_tree);
 #endif
