@@ -657,7 +657,7 @@ scan_linker_output (const char *fname)
       if (! sym && ! end)
 	/* Try a mangled name in quotes.  */
 	{
-	  const char *oldq = q + 1;
+	  char *oldq = q + 1;
 	  demangled *dem = 0;
 	  q = 0;
 
